/*****************************************************
Project : 12 GHz Prescaler :1000
Version :
Date    : 16.05.2011
Author  : Alexander C. Frank
Company : www.changpuak.ch
Comments: Modified for ATMEL STUDIO 6 by Martin Pernter
Chip type  : ATtiny13
AVR Core Clock frequency: 4.000000 MHz

*****************************************************/


#define F_CPU 4.0E6 
#include <avr/io.h>
#include <util/delay.h>



#define SET_CLK (PORTB |= (1 << PB3))
#define CLR_CLK (PORTB &= ~(1 << PB3))
#define SET_DTA (PORTB |= (1 << PB0))
#define CLR_DTA (PORTB &= ~(1 << PB0))

#define SET_LE (PORTB |= (1 << PB1))
#define CLR_LE (PORTB &= ~(1 << PB1))

#define SET_CE (PORTB |= (1 << PB2))
#define CLR_CE (PORTB &= ~(1 << PB2))

unsigned char data[24];
unsigned char i;


int main(void)
{
	
   //#pragma optsize-
   CLKPR=0x80;
   CLKPR=0x00;
   //#ifdef _OPTIMIZE_SIZE_
   //#pragma optsize+
   //#endif

   // Input/Output Ports initialization
   // Port B initialization
   // Func5=In Func4=In Func3=Out Func2=Out Func1=Out Func0=Out
   // State5=T State4=T State3=0 State2=0 State1=0 State0=0
   PORTB=0x00;
   DDRB=0x0F;

   // Timer/Counter 0 initialization
   // Clock source: System Clock
   // Clock value: Timer 0 Stopped
   // Mode: Normal top=FFh
   // OC0A output: Disconnected
   // OC0B output: Disconnected
   TCCR0A=0x00;
   TCCR0B=0x00;
   TCNT0=0x00;
   OCR0A=0x00;
   OCR0B=0x00;

   // External Interrupt(s) initialization
   // INT0: Off
   // Interrupt on any change on pins PCINT0-5: Off
   GIMSK=0x00;
   MCUCR=0x00;

   // Timer/Counter 0 Interrupt(s) initialization
   TIMSK0=0x00;

   // Analog Comparator initialization
   // Analog Comparator: Off
   ACSR=0x80;
   ADCSRB=0x00;

   // METHOD 1 as described in the Datasheet

   CLR_CE;
   CLR_LE;
   CLR_DTA;
   CLR_CLK;

   // Programm the Function LATCH

   // N == 125 = B * P + A
   // P = 8/9
   // B = 15
   // A = 5

   data[23] = 0; // P= 8/9
   data[22] = 0;

   data[21] = 0;
   data[20] = 0;
   data[19] = 0;
   data[18] = 0;

   data[17] = 0;
   data[16] = 0;

   data[15] = 0;
   data[14] = 0;
   data[13] = 0;
   data[12] = 0;
   data[11] = 0;
   data[10] = 0;
   data[9] = 0;

   data[8] = 0;

   data[7] = 0;

   data[6] = 0;
   data[5] = 1; // N-Divider Output
   data[4] = 0;

   data[3] = 0;

   data[2] = 0;

   data[1] = 1; // Function latch
   data[0] = 0;


   for (i=24; i>0 ; i--)
   {
	   if (data[i-1]==1) SET_DTA;
	   else CLR_DTA;
	   _delay_ms(1);
	   SET_CLK;
	   _delay_ms(1);
	   CLR_CLK;
	   _delay_ms(1);
   };

   SET_LE;
   _delay_ms(1);
   CLR_LE;


   // Programm the R-COUNTER (any value :-)

   data[23] = 0;
   data[22] = 0;
   data[21] = 0;
   data[20] = 0;
   data[19] = 0;
   data[18] = 0;

   data[17] = 1;
   data[16] = 0;

   data[15] = 1;
   data[14] = 1;
   data[13] = 1;
   data[12] = 1;
   data[11] = 1;
   data[10] = 1;
   data[9] = 1;
   data[8] = 1;
   data[7] = 1;
   data[6] = 1;
   data[5] = 1;
   data[4] = 1;
   data[3] = 1;
   data[2] = 1;

   data[1] = 0;
   data[0] = 0;


   for (i=24; i>0 ; i--)
   {
	   if (data[i-1]==1) SET_DTA;
	   else CLR_DTA;
	   _delay_ms(1);
	   SET_CLK;
	   _delay_ms(1);
	   CLR_CLK;
	   _delay_ms(1);
   };

   SET_LE;
   _delay_ms(1);
   CLR_LE;


   // Programm the AB-COUNTER

   data[23] = 0;
   data[22] = 0;
   data[21] = 0;

   data[20] = 0;
   data[19] = 0;
   data[18] = 0;
   data[17] = 0;
   data[16] = 0;
   data[16] = 0;
   data[15] = 0;
   data[14] = 0;
   data[13] = 0;
   data[12] = 0;
   data[11] = 1;
   data[10] = 1;
   data[9] = 1;
   data[8] = 1;

   data[7] = 0;
   data[6] = 0;
   data[5] = 0;
   data[4] = 1;
   data[3] = 0;
   data[2] = 1;

   data[1] = 0;
   data[0] = 1;	//N counter (A and B)

   for (i=24; i>0 ; i--)
   {
	   if (data[i-1]==1) SET_DTA;
	   else CLR_DTA;
	   _delay_ms(1);
	   SET_CLK;
	   _delay_ms(1);
	   CLR_CLK;
	   _delay_ms(1);	 	   
   };   
 
    SET_LE;
    _delay_ms(1);
    CLR_LE;

   SET_CE;

   _delay_ms(1000.0);

   while (1)
   {	
	   // DO NOTHING
   };
}